"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _less = require("less");

var _less2 = _interopRequireDefault(_less);

var _pify = require("pify");

var _pify2 = _interopRequireDefault(_pify);

var _processResult = require("./processResult");

var _processResult2 = _interopRequireDefault(_processResult);

var _getOptions = require("./getOptions");

var _getOptions2 = _interopRequireDefault(_getOptions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fs = require("fs");
var md5 = require("md5");
var removeSourceMappingUrl = require("./removeSourceMappingUrl");
var formatLessError = require("./formatLessError");
var debounce = require("lodash/debounce");

var CACHE_FILE_PATH = `${process.env.HOME || process.env.USERPROFILE || "."}/.less-loader-cache.json`;

console.log(CACHE_FILE_PATH);

var render = (0, _pify2.default)(_less2.default.render.bind(_less2.default));

var cache = {};

var cacheHit = 0;
var cacheMiss = 0;

function loadCash() {
  try {
    cache = JSON.parse(fs.readSync(CACHE_FILE_PATH));
  } catch (e) {
    console.err("Error loading cache:", e);
  }
}

function lessLoader(source) {
  // console.log("Source:", source);
  // console.log("Cache:", JSON.stringify(cache, 0, 2));
  var loaderContext = this;
  var options = (0, _getOptions2.default)(loaderContext);
  var done = loaderContext.async();
  var isSync = typeof done !== "function";

  if (isSync) {
    throw new Error("Synchronous compilation is not supported anymore. See https://github.com/webpack-contrib/less-loader/issues/84");
  }
  var callback = loaderContext.callback;


  var cachedObj = getFromCache(source);
  if (!cachedObj) {
    cacheMiss++;
    var resultPromise = render(source, options);
    resultPromise.then(function (_ref) {
      var css = _ref.css,
          map = _ref.map,
          imports = _ref.imports;

      imports.forEach(loaderContext.addDependency, loaderContext);
      return {
        css: removeSourceMappingUrl(css),
        map: typeof map === "string" ? JSON.parse(map) : map,
        imports
      };
    }, function (lessError) {
      var error = formatLessError(lessError);
      storeCache(source, { css: null, map: null, error });
      throw error;
    }).then(function (_ref2) {
      var css = _ref2.css,
          map = _ref2.map,
          imports = _ref2.imports;

      storeCache(source, { css, map, imports, error: null });
      debouncedWriteCache();
      callback(null, css, map);
    }, callback);
  } else {
    cacheHit++;
    if (cachedObj.error) {
      throw cachedObj.error;
    } else {
      var imports = cachedObj.imports;
      imports.forEach(loaderContext.addDependency, loaderContext);
      callback(null, cachedObj.css, cachedObj.map);
    }
  }
  console.log("Hit:", cacheHit, "Miss:", cacheMiss);
}

function storeCache(source, cacheObj) {
  var hash = md5(source);
  cache[hash] = cacheObj;
}

function getFromCache(source) {
  var hash = md5(source);
  return cache[hash];
}

function writeCache() {
  try {
    fs.writeFile(CACHE_FILE_PATH, JSON.stringify(cache, null, 2), console.err);
  } catch (e) {
    console.log(e);
  }
}

var debouncedWriteCache = debounce(writeCache, 0);

exports.default = lessLoader;